/* 
 * Plugin Name: Statistics - nPVI moving average (nPVIMA)
 * Plugin Description: Computes nPVI (Normalized Pairwise Variability Index, as in  Grabe & Low, 2002) moving average values
 * Plugin Author: Katarzyna Klessa & Wojciech Klessa
 * Plugin Created: 2014-01-02
 * 
 * Plugin Id: faa5093d
 * Plugin Version: 1.0
 * 
 */

using System;
using System.Windows.Forms;
using AnnotationPro.Plugin;
using AnnotationPro.Logic;
using AnnotationPro.Presentation;
using AnnotationPro.Statistics;
using System.Collections.Generic;

namespace AnnotationPro.Plugin
{
    public class AnnotationPlugin : IAnnotationPlugin
    {
        public void Run(AnnotationEditor editor)
        {
            // BEGIN USER CODE

            string layerName = "SRC";
            float frameWidth = 5;
            float frameStep = frameWidth * 2 / 3; // 33%

            Annotation annotation = editor.Synchronizer.DataServer.Annotation;
            Audio audio = editor.Synchronizer.DataServer.Audio;

            LayerObject srcLayer = null;
            foreach (LayerObject layer in annotation.Layers)
            {
                if (layer.Name == layerName)
                {
                    srcLayer = layer;
                    break;
                }
            }
            // Use this line if you want to select layer by selecting it by clicking on the interface and not by its name:           
            // LayerObject regionsLayer = annotation.Layers.FindByName( "Regions" );
            // Info when the source layer is missing:
            if (srcLayer == null)
            {
                MessageBox.Show("Layer is missing: " + layerName);
                return;
            }

            LayerObject dstLayer = editor.AnnotationLayers.AddNewLayer();
            dstLayer.Name = srcLayer.Name + " - nPVI MA";
            dstLayer.Parameter1Name = "FrameStart + FrameEnd";
            dstLayer.Parameter2Name = "Count";
            dstLayer.Parameter3Name = "nPVI";

            float maxTime = audio.SamplesCount / audio.Samplerate;
            int frameIndex = 0;
            for (float frameOffset = 0; frameOffset < maxTime; frameOffset += frameStep)
            {
                float frameStart = frameOffset * annotation.Samplerate;
                float frameEnd = (frameOffset + frameWidth) * annotation.Samplerate;
                float frameMiddle = (frameOffset + frameWidth / 2) * annotation.Samplerate;

                // count how many segments are included in the frame and how much time they last
                float count = 0;
                float countWithFractions = 0;
                float totalDuration = 0;
                List<double> durations = new List<double>();

                foreach (SegmentObject segment in srcLayer.Segments)
                {
                    if (segment.End <= frameStart || segment.Start > frameEnd) continue;
                    if (segment.Label == "#$p") continue; // each line excludes a label from computations
                    if (segment.Label == "[fp]") continue;
                    if (segment.Label == "...") continue;
                    count++;
                    // count how many segments with fractions (parts of segments) are included in the frame and how much time they last
                    float intersection = GetSegmentIntersection(segment, frameStart, frameEnd);
                    float fraction = intersection / segment.Duration;

                    durations.Add(segment.Duration / annotation.Samplerate);

                    countWithFractions += fraction;
                    totalDuration += intersection / annotation.Samplerate;
                }

                // adding a segment with the results in the middle of the frame as the segment label (the segment width is a half of frame width)
                SegmentObject dstSegment = new SegmentObject();
                dstSegment.Start = frameStart;
                dstSegment.Duration = 0.5f * annotation.Samplerate;

                dstSegment.Label = "Frame #" + frameIndex + " Count: " + count.ToString() + ", Count (FRACT): " + countWithFractions.ToString() + ", Total Dur: " + totalDuration.ToString() + ", Frame Width:" + frameWidth;
                float frameOffsetPlusWidth = frameOffset + frameWidth;

                dstSegment.Parameter1 = frameOffset.ToString() + "+" + frameOffsetPlusWidth.ToString();
                dstSegment.Parameter2 = count.ToString();
                double nPVI = GeneralStatistics.nPVI(durations.ToArray());
                dstSegment.Parameter3 = nPVI.ToString();

                dstLayer.Segments.Add(dstSegment);

                frameIndex++;
            }

            // END USER CODE
        }

        public float GetSegmentIntersection(SegmentObject segment, float rangeStart, float rangeEnd)
        {
            if (segment.Start < rangeStart && segment.End > rangeStart && segment.End <= rangeEnd)
            {
                return segment.End - rangeStart;
            }
            if (segment.Start < rangeEnd && segment.Start > rangeStart && segment.End > rangeEnd)
            {
                return rangeEnd - segment.Start;
            }
            if (segment.Start > rangeStart && segment.End < rangeEnd)
            {
                return segment.Duration;
            }
            if (segment.Start < rangeStart && segment.End > rangeEnd)
            {
                return rangeEnd = rangeStart;
            }

            return 0;
        }
    }
}